#include "BNO055.h"

// Configure the BNO055
void config_BNO055()
{
    I2C5_init(25500);
    
    uint8_t read_byte;
    
    // Read device ID = 0xA0 to ensure I2C is working
    I2C5_start();
    I2C5_write(ADDR_W_BNO);
    I2C5_write(0x00);
    I2C5_stop();
    
    I2C5_start();
    I2C5_write(ADDR_R_BNO);
    read_byte = I2C5_read(1);
    I2C5_stop();
 
    
    // Axis remap - AXIS_REMAP_CONFIG and AXIS_REMAP_SIGN (Not implemented)
    
    
    // Set config to NDOF mode - OPR_MODE = 0x0C 
    I2C5_start();
    I2C5_write(ADDR_W_BNO);
    I2C5_write(0x3D);
    I2C5_write(0x0C);
    I2C5_stop();
    
    // Delay at least 6ms to allow mode change
    delay();
}


void read_BNO055(Data_BNO* data)
{
    // Initialize variables to store the I2C reads
    uint16_t eul_pitch_msb = 0;
    uint16_t eul_pitch_lsb = 0;
    uint16_t eul_roll_msb = 0;
    uint16_t eul_roll_lsb = 0;
    uint16_t eul_heading_msb = 0;
    uint16_t eul_heading_lsb = 0;
    
    uint16_t gyr_z_msb = 0;
    uint16_t gyr_z_lsb = 0;
    uint16_t gyr_y_msb = 0;
    uint16_t gyr_y_lsb = 0;
    uint16_t gyr_x_msb = 0;
    uint16_t gyr_x_lsb = 0;
    
    uint16_t mag_z_msb = 0;
    uint16_t mag_z_lsb = 0;
    uint16_t mag_y_msb = 0;
    uint16_t mag_y_lsb = 0;
    uint16_t mag_x_msb = 0;
    uint16_t mag_x_lsb = 0;
    
    uint16_t accel_z_msb = 0;
    uint16_t accel_z_lsb = 0;
    uint16_t accel_y_msb = 0;
    uint16_t accel_y_lsb = 0;
    uint16_t accel_x_msb = 0;
    uint16_t accel_x_lsb = 0;
    
    
    // Initialize variables to store the combined MSBs and LSBs    
    int16_t eul_pitch = 0;
    int16_t eul_roll = 0;
    int16_t eul_heading = 0;
    
    int16_t gyr_x = 0;
    int16_t gyr_y = 0;
    int16_t gyr_z = 0;
    
    int16_t mag_x = 0;
    int16_t mag_y = 0;
    int16_t mag_z = 0;
    
    int16_t accel_x = 0;
    int16_t accel_y = 0;
    int16_t accel_z = 0;
    
    
    // Read data starting at address 0x08 - 0x27
    I2C5_start();
    I2C5_write(ADDR_W_BNO);
    I2C5_write(0x08);
    I2C5_stop();
    
    I2C5_start();
    I2C5_write(ADDR_R_BNO);
    
    accel_x_lsb = I2C5_read(0);
    accel_x_msb = I2C5_read(0);
    accel_y_lsb = I2C5_read(0);
    accel_y_msb = I2C5_read(0);
    accel_z_lsb = I2C5_read(0);
    accel_z_msb = I2C5_read(0);
    
    mag_x_lsb = I2C5_read(0);
    mag_x_msb = I2C5_read(0);
    mag_y_lsb = I2C5_read(0);
    mag_y_msb = I2C5_read(0);
    mag_z_lsb = I2C5_read(0);
    mag_z_msb = I2C5_read(0);
    
    gyr_x_lsb = I2C5_read(0);
    gyr_x_msb = I2C5_read(0);
    gyr_y_lsb = I2C5_read(0);
    gyr_y_msb = I2C5_read(0);
    gyr_z_lsb = I2C5_read(0);
    gyr_z_msb = I2C5_read(0);
    
    eul_heading_lsb = I2C5_read(0);
    eul_heading_msb = I2C5_read(0);
    eul_roll_lsb = I2C5_read(0);
    eul_roll_msb = I2C5_read(0);
    eul_pitch_lsb = I2C5_read(0);
    eul_pitch_msb = I2C5_read(1);
    
    I2C5_stop();
    
    
    // Combine MSBs and LSBs
    eul_pitch = (eul_pitch_msb << 8) | eul_pitch_lsb;
    eul_roll = (eul_roll_msb << 8) | eul_roll_lsb;
    eul_heading = (eul_heading_msb << 8) | eul_heading_lsb;
    
    gyr_x = (gyr_x_msb << 8) | gyr_x_lsb;
    gyr_y = (gyr_y_msb << 8) | gyr_y_lsb;
    gyr_z = (gyr_z_msb << 8) | gyr_z_lsb;
    
    mag_x = (mag_x_msb << 8) | mag_x_lsb;
    mag_y = (mag_y_msb << 8) | mag_y_lsb;
    mag_z = (mag_z_msb << 8) | mag_z_lsb;
    
    accel_x = (accel_x_msb << 8) | accel_x_lsb;
    accel_y = (accel_y_msb << 8) | accel_y_lsb;
    accel_z = (accel_z_msb << 8) | accel_z_lsb;
    
    
    // Convert data to floats and write to data struct    
    data->eul_pitch_data = ((float)eul_pitch / EULDIV);
    data->eul_roll_data = ((float)eul_roll / EULDIV);
    data->eul_heading_data = ((float)eul_heading / EULDIV);
    
    data->gyr_x_data = ((float)gyr_x / GYRDIV);
    data->gyr_y_data = ((float)gyr_y / GYRDIV);
    data->gyr_z_data = ((float)gyr_z / GYRDIV);
    
    data->mag_x_data = ((float)mag_x / MAGDIV);
    data->mag_y_data = ((float)mag_y / MAGDIV);
    data->mag_z_data = ((float)mag_z / MAGDIV);
    
    data->accel_x_data = ((float)accel_x / ACCELDIV);
    data->accel_y_data = ((float)accel_y / ACCELDIV);
    data->accel_z_data = ((float)accel_z / ACCELDIV);
}

// It is recommended to wait after configuring the BNO055 before reading data
void delay(void)
{
    int i;
    for (i=0; i<100000; i++)
    {
        
    }
}