#include <xc.h>
#include "i2c5mz.h" 

void I2C5_init(unsigned long rate){
    ETHCON1bits.ON = 0; // disable ethernet to ensure no collisions
    
    rate = 100000;
	
    // Disable interrupts
    IEC5bits.I2C5BIE = 0;
    IEC5bits.I2C5MIE = 0;
    IEC5bits.I2C5SIE = 0;
        
    /*
    // set the clock rate based on user input - Note: we decided to hard code this to avoid bugs
    unsigned long pbclk = 25000000; // 25 MHz
    I2C5BRG = pbclk/(2*rate) - 2; // this ignores pulse gobble
    */
	
    I2C5BRG = 0x060; // PIC32 Reference Section 24

    // turn off for config
    I2C5CONbits.ON = 0;
    
    if(rate < 200000){
        I2C5CONbits.DISSLW = 1;
    }
    else{
        I2C5CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C5CONbits.ON = 1;
    MIF5 = 0;    // clear master interrupt flag
    
}


void I2C5_start(void){
    I2C5CONbits.SEN = 1;      	// Initialize start
	while(!MIF5){}            	// Wait until start finished
    MIF5 = 0; 					// Clear MI flag
 
}


void I2C5_stop(void){
    I2C5CONbits.PEN=1;       	// Initiate stop, PEN=1
    while(!MIF5){}            	// Wait until stop finished
	MIF5 = 0;
}


void I2C5_restart(void){
    I2C5CONbits.RSEN = 1;		// Repeat start
	while(!MIF5);				// Wait until start finished
    MIF5 = 0;
}


uint8_t I2C5_write(uint8_t data) {
    I2C5TRN = data;                     // Put data in transmit buffer
    while(!MIF5){}        				// Wait until transfer finished
    MIF5 = 0;
    return(I2C5STATbits.ACKSTAT);       // Return ACK         
}


uint8_t I2C5_read(int8_t ack2send) {
    unsigned char inByte;
    
    I2C5CONbits.RCEN = 1;               // Enable receive
    while(!MIF5){}          			// Wait for receive byte
    MIF5 = 0;

    inByte = I2C5RCV;                   // Read byte    
    I2C5_ACK(ack2send);					// Send ACK/NACK signal
    return(inByte);						// Return received byte
	
}


void I2C5_ACK(int8_t ack2send){
	I2C5CONbits.ACKDT = ack2send;       // ACK type to send (ACK or NACK)
    I2C5CONbits.ACKEN = 1;              // Enable ACK bit transmission
    while(!MIF5){}          			// Wait until done
	MIF5 = 0;
}
