#include "KX222.h"

void config_KX222()
{
    I2C1_init();
    I2C1_start();
    
    uint8_t config;
    
    // CNTL1 Register - Put chip in standby mode to write to registers
    I2C1_write(ADDR_W);
    I2C1_write(0x18);
    I2C1_write(0x00);
    I2C1_stop();
    Nop();
    
    // Determine chip config based on max acceleration
    switch (MODE)
    {
        case 32:
            config = 0xF0;
            break;
        
        case 16:
            config = 0xE8;
            break;
            
        case 8:
            config = 0xE0;
            break;
            
        default:
            config = 0xF0;
    }
    
    // ODCNTL Register - Set output data rate to 800Hz
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x1B);
    I2C1_write(0x06);
    
    // INC5 and INC6 Registers - Set up data ready interrupt on INT2 pin
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x20);
    I2C1_write(0x3A);
    I2C1_write(0x10);
    
    // CNTL1 Register - Put chip in operating mode and set up parameters
    I2C1_restart();
    I2C1_write(ADDR_W);
    I2C1_write(0x18);
    I2C1_write(config);
    I2C1_stop();
    
}


void read_KX222(Data_KX* data)
{
    uint16_t x_lsb = 0;
    uint16_t x_msb = 0;
    uint16_t y_lsb = 0;
    uint16_t y_msb = 0;
    uint16_t z_lsb = 0;
    uint16_t z_msb = 0;
    int16_t x_accel = 0;
    int16_t y_accel = 0;
    int16_t z_accel = 0;
    float max_value = 32768;
    
    // Start reading acceleration data
    I2C1_start();
    I2C1_write(ADDR_W);
    I2C1_write(0x06);
    
    I2C1_restart();
    I2C1_write(ADDR_R);
    x_lsb = I2C1_read(0);
    x_msb = I2C1_read(0);
    y_lsb = I2C1_read(0);
    y_msb = I2C1_read(0);
    z_lsb = I2C1_read(0);
    z_msb = I2C1_read(1);
    I2C1_stop();
    
    // Combine msb and lsb for each reading
    x_accel = (x_msb << 8) | x_lsb;
    y_accel = (y_msb << 8) | y_lsb;
    z_accel = (z_msb << 8) | z_lsb;
    
    // Store acceleration values in data struct
    data->x_data = (float)MODE*((float)x_accel / max_value);
    data->y_data = (float)MODE*((float)y_accel / max_value);
    data->z_data = (float)MODE*((float)z_accel / max_value);
}



void I2C1_init(unsigned long rate){
    ETHCON1bits.ON = 0;
    
    rate = 100000;
    // Disable interrupts
    IEC3bits.I2C1BIE = 0;
    IEC3bits.I2C1MIE = 0;
    IEC3bits.I2C1SIE = 0;
        
    /*
    // set the clock rate based on user input
    unsigned long pbclk = 25000000; // 25 MHz
    I2C15BRG = pbclk/(2*rate) - 2; // this ignores pulse gobble
    */
    I2C1BRG = 0x060; // PIC32 Reference Section 24

    // turn off for config
    I2C1CONbits.ON = 0;
    
    if(rate < 200000){
        I2C1CONbits.DISSLW = 1;
    }
    else{
        I2C1CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C1CONbits.ON = 1;
    MIF1 = 0;    // clear flag
    
}


void I2C1_start(void){
    
    // we can experiment with robustness later
    I2C1CONbits.SEN = 1;                // initialize start
	while(!MIF1){}            // wait until start finished
    MIF1 = 0; // clear flag
 
}


void I2C1_stop(void){
    I2C1CONbits.PEN=1;                  // initiate stop, PEN=1
    while(!MIF1){}            // wait until stop finished
	MIF1 = 0;
}


void I2C1_restart(void){
    I2C1CONbits.RSEN = 1;
	while(!MIF1);
    MIF1 = 0;
}


uint8_t I2C1_write(uint8_t data) {
    I2C1TRN = data;                     // put data in transmit buffer
    while(!MIF1){}        // wait until transfer finished
    MIF1 = 0;
    return(I2C1STATbits.ACKSTAT);       // return ACK         
}


uint8_t I2C1_read(int8_t ack2send) {
    unsigned char inByte;
    //while(I2C15CON & 0x1F){}             // wait for idle condition (nothing in progress)
    
    
    I2C1CONbits.RCEN = 1;               // enable receive
    while(!MIF1){}          // wait for receive byte
    MIF1 = 0;
    inByte = I2C1RCV;                   // read byte
    //while(I2C15CON & 0x1F){}             // wait for idle
    
    I2C1_ACK(ack2send);					// send ACK signal
    return(inByte);						// return received byte
	
}


void I2C1_ACK(int8_t ack2send){
	I2C1CONbits.ACKDT = ack2send;       // ACK type to send
    I2C1CONbits.ACKEN = 1;              // enable ACK bit transmission
    while(!MIF1){}          // wait until done
	MIF1 = 0;
}
