#include "adf7030_tools.h"
#include "spi_tools.h"


#include "CS_def_sensor_board.h"
//#include "CS_def_NOT_sensor_board.h"

void reset_radio()
{
    ANSELBbits.ANSB6 = 0;
    TRISBbits.TRISB6 = 0;
    
    TRISBbits.TRISB6 = 0;
    LATBbits.LATB6 = 1;
    LATBbits.LATB6 = 0;
    int i;
    for (i=0; i<10000; i++);
    LATBbits.LATB6 = 1; 
}

uint32_t read_RnPL(uint32_t address)                        //random, no pointer, long address
{    
    nCS = 0;
    spi_write(0b01011000);
       
    spi_write( (uint8_t)((address & 0xFF000000) >> 24) );
    spi_write( (uint8_t)((address & 0x00FF0000) >> 16) );
    spi_write( (uint8_t)((address & 0x0000FF00) >> 8) );
    spi_write( (uint8_t)((address & 0x000000FF) ) );
    
    spi_write(0x00);
    spi_write(0x00);
    
    uint32_t Value = 0x00000000;
    Value = ( (uint32_t)spi_write(0x00) << 24);
    Value = Value | (( (uint32_t)spi_write(0x00) << 16) & 0x00FF0000);
    Value = Value | (( (uint32_t)spi_write(0x00) << 8) & 0x0000FF00);
    Value = Value | ((uint32_t)spi_write(0x00) & 0x000000FF);

    nCS = 1;
    
    return Value;
    
}


void write_RnPL(uint32_t address, uint32_t data)            //random, no pointer, long address
{    
    nCS = 0;
    spi_write(0b00011000);
       
    spi_write( (uint8_t)((address & 0xFF000000) >> 24) );
    spi_write( (uint8_t)((address & 0x00FF0000) >> 16) );
    spi_write( (uint8_t)((address & 0x0000FF00) >> 8) );
    spi_write( (uint8_t)((address & 0x000000FF) ) );
    
    
    spi_write( (uint8_t)((data & 0xFF000000) >> 24) );
    spi_write( (uint8_t)((data & 0x00FF0000) >> 16) );
    spi_write( (uint8_t)((data & 0x0000FF00) >> 8) );
    spi_write( (uint8_t)((data & 0x000000FF) ) );    

    nCS = 1;
    
    
}

void set_pntr(uint32_t address, uint8_t pntr_num)
{   
    if (pntr_num == 0)
    {
        write_RnPL(SPI_HOST_PNTR0, address);
    }
    else if (pntr_num == 1)
    {
        write_RnPL(SPI_HOST_PNTR1, address);
    }
    else if (pntr_num == 2)
    {
        write_RnPL(SPI_HOST_PNTR2, address);
    }
}

void read_BPL(uint8_t pntr_num, uint8_t* buff, uint32_t size)      //block, pointer, long address
{
    uint8_t pntr_val;
    switch (pntr_num)
    {
        case 0: pntr_val = 0b101;
            break;
        case 1: pntr_val = 0b110;
            break;
        case 2: pntr_val = 0b111;
            break;
        default: return;
    }
    
    nCS = 0;
    spi_write(0x68 | pntr_val);
    uint32_t i;    
    spi_write(0x00);
    spi_write(0x00);
    for(i = 0; i < size; i++)
    {
        *buff = spi_write(0x00);
        buff++;
    }
    nCS = 1;
}

void write_BPL(uint8_t pntr_num, uint8_t* buff, uint32_t size)      //block, pointer, long address
{
    uint8_t pntr_val;
    switch (pntr_num)
    {
        case 0: pntr_val = 0b101;
            break;
        case 1: pntr_val = 0b110;
            break;
        case 2: pntr_val = 0b111;
            break;
        default: return;
    }
    
    nCS = 0;
    spi_write(0x28 | pntr_val);
    uint32_t i;    
    for(i = 0; i < size; i++)
    {
        spi_write(*buff);
        buff++;
    }
    nCS = 1;
}


void poll_CMDREADY_IDLE()
{
    nCS = 0;
    uint8_t status = spi_write(0xFF);
    status = spi_write(0xFF);
    while(!((status&0x20) && (status&0x04))){
        status = spi_write(0xFF);
    } 
    nCS = 1;
}

uint8_t is_CMDREADY_IDLE()
{
    nCS = 0;
    uint8_t status = spi_write(0xFF);
    status = spi_write(0xFF);
    nCS = 1;
    
    if ( (status&0x20) && (status&0x04) ) {
        return 1;
    }
    else{
        return 0;
    }
}


// Configures radio parameters using cfg file
void configRadio()
{
    nCS = 0;
    const uint8_t Radio_Memory_Configuration[ ] = {
//        #include "feb15_payload100B_datarate250_PA2.cfg"
//        #include "feb2_payload100B_datarate250kbps.cfg"
//        #include "power_test_250kbps_PA2_13dBm.cfg"
//        #include "feb19_PA2_test.cfg"
        #include "feb20_250kbps_PA2_17dBm.cfg"
    };
    unsigned int i = 0;
    unsigned int j;
    unsigned int block_size = 0;
    unsigned int raw_size;
    unsigned int lengthCfg = sizeof(Radio_Memory_Configuration)/sizeof(Radio_Memory_Configuration[0]);
    do{
        raw_size = 0;
        for (j = 0; j < 3; j++)
        {
            raw_size = raw_size | (unsigned int)(Radio_Memory_Configuration[i] << (8*(2-j)));
            i++;
        }
        block_size = raw_size - 3;
        nCS = 0;
        for (j = 0; j < block_size; j++)
        {
            spi_write(Radio_Memory_Configuration[i]);
            i++;
        }
        nCS = 1;
    } while (i < lengthCfg);
}


// Configures radio parameters using cfg file
void calibrateRadio()
{
    nCS = 0;
    const uint8_t Radio_Memory_Calibration[ ] = {
        #include "OffLineCalibrations.cfg"
    };
    unsigned int i = 0;
    unsigned int j;
    unsigned int block_size = 0;
    unsigned int raw_size;
    unsigned int lengthCfg = sizeof(Radio_Memory_Calibration)/sizeof(Radio_Memory_Calibration[0]);
    do{
        raw_size = 0;
        for (j = 0; j < 3; j++)
        {
            raw_size = raw_size | (unsigned int)(Radio_Memory_Calibration[i] << (8*(2-j)));
            i++;
        }
        block_size = raw_size - 3;
        nCS = 0;
        for (j = 0; j < block_size; j++)
        {
            spi_write(Radio_Memory_Calibration[i]);
            i++;
        }
        nCS = 1;
    } while (i < lengthCfg);
}



void radio_command(uint8_t state)
{
    nCS = 0;
    spi_write(0x80 | state);
    nCS = 1;
}

// Enables SPI commands to be sent to radio
void enableSPICMD()
{
        // Make MISO pin an input
    TRISBbits.TRISB9 = 1;
         // Make SS pin an output
//    TRISBbits.TRISB7 = 0;
    TRIS_CS = 0;
    // Pull SS pin low
//    LATBbits.LATB7 = 0;
    LAT_CS = 0;
    

    while(!PORTBbits.RB9){}     // Wait for MISO pin to go high
    
//    LATBbits.LATB7 = 1;         // Pull SS pin high
    LAT_CS = 1;
}

void write_bits(uint32_t address, uint8_t numbits, uint8_t lowidx, uint32_t data)
{
    uint32_t val = read_RnPL(address);   
    uint8_t i;
    uint32_t mask = 0;
    for (i = 0; i < numbits; i++)
    {
        mask = mask << 1;
        mask += 1;
    }
    mask = mask << lowidx;
    uint32_t notmask = ~mask;
    val = val & notmask;            // clear desired bits
    val = val | (data << lowidx);   // overwrite with desired data
    write_RnPL(address, val);
}

uint32_t read_bits(uint32_t address, uint8_t numbits, uint8_t lowidx)
{
    uint32_t data;
    uint32_t val = read_RnPL(address);
    uint8_t i;
    uint32_t mask = 0;
    for (i = 0; i < numbits; i++)
    {
        mask = mask << 1;
        mask += 1;
    }
    mask = mask << lowidx;
    data = (val & mask) >> lowidx;
    return data;
}


void setup_adf_tx()
{
    reset_radio();      
    enableSPICMD();                                                             //Enables SPI commands to be sent to radio
    spi_init();                                                                 //Enable PIC SPI
     
    poll_CMDREADY_IDLE();                                                       //Configure ADF
    configRadio();                                                              //Configure ADF

    write_bits(GENERIC_PKT_BUFF_CFG1,9,0,TX_SIZE);                              //Set TX_SIZE = 100
    write_bits(GENERIC_PKT_FRAME_CFG1,12,0,TX_SIZE);                            // Set Payload Size to TX_SIZE
    
    write_bits(PROFILE_REF_CLK_CFG,1,26,1);                                     //Modify configuration such that CLK_TYPE = 1 (oscillator is XTAL)
    calibrateRadio();                                                           //Write calibration file to memory
    write_RnPL(SM_DATA_CALIBRATION, CAL_ENABLE);
    radio_command(CFG_DEV);                                                     //Send command to do configuration
    poll_CMDREADY_IDLE();                                                       //Wait for CMD_READY, TRANSITION_STATE==IDLE
    radio_command(PHY_ON);                                                      //Sequence to perform calibration
    poll_CMDREADY_IDLE(); 
    radio_command(DO_CAL);
    poll_CMDREADY_IDLE(); 
    write_RnPL(SM_DATA_CALIBRATION, CAL_DISABLE);
    
    uint32_t PTR_TX = read_bits(GENERIC_PKT_BUFF_CFG0,11,11)*4 + 0x20000000;    //READ PTR_TX
    set_pntr(PTR_TX,0);                                                         //set pointer 0 as TX payload pointer
}