#include <xc.h>
#include "i2c4mz.h" 

void I2C4_init(unsigned long rate){
    CM1CONbits.ON = 0; // disable comparator 1
    CM2CONbits.ON = 0; // disable comparator 2
    ANSELGbits.ANSG8 = 0; // disable pin 6 analog function (SCL4)
    ANSELGbits.ANSG7 = 0; // disable pin 5 analog function (SDA4)
    
    rate = 100000;
    // Disable interrupts
    IEC5bits.I2C4BIE = 0;
    IEC5bits.I2C4MIE = 0;
    IEC5bits.I2C4SIE = 0;
        
    /*
    // set the clock rate based on user input
    unsigned long pbclk = 25000000; // 25 MHz
    I2C5BRG = pbclk/(2*rate) - 2; // this ignores pulse gobble
    */
    I2C4BRG = 0x060; // PIC32 Reference Section 24

    // turn off for config
    I2C4CONbits.ON = 0;
    
    if(rate < 200000){
        I2C4CONbits.DISSLW = 1;
    }
    else{
        I2C4CONbits.DISSLW = 0;
    }
    
    // turn on after config
    I2C4CONbits.ON = 1;
    MIF4 = 0;    // clear flag
    
}


void I2C4_start(void){
    
    // we can experiment with robustness later
    I2C4CONbits.SEN = 1;                // initialize start
	while(!MIF4){}            // wait until start finished
    MIF4 = 0; // clear flag
 
}


void I2C4_stop(void){
    I2C4CONbits.PEN=1;                  // initiate stop, PEN=1
    while(!MIF4){}            // wait until stop finished
	MIF4 = 0;
}


void I2C4_restart(void){
    I2C4CONbits.RSEN = 1;
	while(!MIF4);
    MIF4 = 0;
}


uint8_t I2C4_write(uint8_t data) {
    I2C4TRN = data;                     // put data in transmit buffer
    while(!MIF4){}        // wait until transfer finished
    MIF4 = 0;
    return(I2C4STATbits.ACKSTAT);       // return ACK         
}


uint8_t I2C4_read(int8_t ack2send) {
    unsigned char inByte;
    //while(I2C5CON & 0x1F){}             // wait for idle condition (nothing in progress)
    
    
    I2C4CONbits.RCEN = 1;               // enable receive
    while(!MIF4){}          // wait for receive byte
    MIF4 = 0;
    inByte = I2C4RCV;                   // read byte
    //while(I2C5CON & 0x1F){}             // wait for idle
    
    I2C4_ACK(ack2send);					// send ACK signal
    return(inByte);						// return received byte
	
}


void I2C4_ACK(int8_t ack2send){
	I2C4CONbits.ACKDT = ack2send;       // ACK type to send
    I2C4CONbits.ACKEN = 1;              // enable ACK bit transmission
    while(!MIF4){}          // wait until done
	MIF4 = 0;
}
