/* 
 * File:   main.c
 * Author: zacharykowalczyk
 *
 * Created on February 15, 2020, 6:07 PM
 */

#include <stdio.h>
#include <stdlib.h>
#include <xc.h>
#include "SD_interface.h"
#include "pin_config.h"
#include "Adafruit_MPL3115A2.h"
#include "KX222.h"
#include "camm8c010.h"
#include "i2c3mz.h"
#include "spi_tools.h"
#include "adf7030_tools.h"
#include <sys/attribs.h>
#include "CS_def_sensor_board.h"
#include "packet.h"
#include "BNO055.h"

/*
 *  Global Variables
 */

uint8_t payload_buff[] = {
        #include "payload_100B.txt"
    };

Packet packet_A;
Packet packet_B;
Packet* current_packet;

uint32_t time;
uint8_t kx_index;

/*
 *  Interrupt Routines
 */
void __ISR (_TIMER_1_VECTOR, IPL5SRS) 
    Timer1Handler(void)
{      
        uint8_t CMDREADY_IDLE = is_CMDREADY_IDLE();
        if(!CMDREADY_IDLE){
            radio_command(PHY_ON);
        }
        else{
            uint32_t status_4byte = read_RnPL(MISC_FW);
            uint8_t CURR_STATE = (status_4byte & 0x00003F00) >> 8;
            if (CURR_STATE != PHY_ON){
                radio_command(PHY_ON);
            }
            else{
                //Sample BNO
                I2C5_start();
                I2C5_write(ADDR_W_BNO);
                I2C5_write(0x35);
                I2C5_stop();

                I2C5_start();
                I2C5_write(ADDR_R_BNO);
                I2C5_read(1);
                I2C5_stop();
                read_BNO055(&(current_packet->data.bno_data));
                
                // Sample Altimeter
                current_packet->data.altitude = getAltitudeOS();
                
                
                Packet* temp_packet = current_packet;
                kx_index = 0;
                
                // Swap the current packet to the other one before transmission
                if (current_packet == &packet_A){
                    current_packet = &packet_B;
                } 
                else {
                    current_packet = &packet_A;
                }
                
                
                temp_packet->data.time = time;
                
                DEBUG3 = DEBUG_ON;
                //Write data to payload buffer
                write_BPL(0,temp_packet->buffer,TX_SIZE);                              //Write to payload buffer using pointer 0 bc PTR_TX associated w pointer 0
                radio_command(PHY_TX);               
                DEBUG3 = DEBUG_OFF;
                
                //Write to SD card
                char output[5000];
                sprintf(output, "%lu, %f, %f, %f, %f, %f, %f, %f, %f, %f, %f, %f, %f, %f", temp_packet->data.time, temp_packet->data.altitude, temp_packet->data.bno_data.eul_heading_data, temp_packet->data.bno_data.eul_pitch_data, temp_packet->data.bno_data.eul_roll_data, temp_packet->data.bno_data.gyr_x_data, temp_packet->data.bno_data.gyr_y_data, temp_packet->data.bno_data.gyr_z_data, temp_packet->data.bno_data.mag_x_data, temp_packet->data.bno_data.mag_y_data, temp_packet->data.bno_data.mag_z_data, temp_packet->data.bno_data.accel_x_data, temp_packet->data.bno_data.accel_y_data, temp_packet->data.bno_data.accel_z_data);
                int i = 0;
                for (i=0; i < 8; i++){
                    sprintf(output, "%s, %f, %f, %f", output, temp_packet->data.kx_data[i].x_data, temp_packet->data.kx_data[i].y_data, temp_packet->data.kx_data[i].z_data);
                }
                sprintf(output, "%s\n", output);
                SD_write_str(output);
                SD_sync();
            }
        }
        IFS0bits.T1IF = 0;
        TMR1 = 0;
}

// Millisecond Timer Routine
void __ISR (_TIMER_2_VECTOR, IPL7SRS) 
    Timer2Handler(void)
{
    time++;
    IFS0bits.T2IF = 0;
    TMR2 = 0;
}


// KX222 Interrupt Handler (_EXTERNAL_4_VECTOR)
void __ISR (_TIMER_3_VECTOR, IPL6SRS) 
    INT4Handler(void)
{
    DEBUG2 = DEBUG_ON;
    read_KX222(&(current_packet->data.kx_data[kx_index++]));
    IFS0bits.T3IF = 0;
    DEBUG2 = DEBUG_OFF;
}



//Timer1 Set up: ~10ms if PBS is 40MHz
void config_timer1_int()
{   
    IEC0bits.T1IE = 0;      //disable
    IFS0bits.T1IF = 0;      //flag        
    T1CONbits.TON = 0;      //start with timer 1 off
    IPC1bits.T1IP = 5;      //highest priorty  
    T1CONbits.TCS = 0;      //use internal clock    
    T1CONbits.TGATE = 0;    //disable clock gating
    T1CONbits.TCKPS = 0b10; //prescaler set to 1:64
    TMR1 = 0;               //reset timer 1
    PR1 = 2000;             //timer 1 period value
    T1CONbits.TON = 1;      //turn on
    IEC0bits.T1IE = 1;      //enable   
}


void config_timer2_int()
{
    IEC0bits.T2IE = 0;      //disable
    IFS0bits.T2IF = 0;      //flag        
    T2CONbits.TON = 0;      //start with timer 1 off
    IPC2bits.T2IP = 7;      //highest priorty  
    T2CONbits.TCS = 0;      //use internal clock    
    T2CONbits.TGATE = 0;    //disable clock gating
    T2CONbits.TCKPS = 0b010; //prescaler set to 4
    TMR2 = 0;               //reset timer 1
    PR2 = 3125;             //timer 1 period value
    T2CONbits.TON = 1;      //turn on
    time = 0;
    IEC0bits.T2IE = 1;      //enable
}


void config_timer3_int()
{
    IEC0bits.T3IE = 0;      //disable
    IFS0bits.T3IF = 0;      //flag        
    T3CONbits.TON = 0;      //start with timer 1 off
    IPC3bits.T3IP = 6;      //highest priorty  
    T3CONbits.TCS = 0;      //use internal clock    
    T3CONbits.TGATE = 0;    //disable clock gating
    T3CONbits.TCKPS = 0b000; //prescaler set to 1
    TMR3 = 0;               //reset timer 1
    PR3 = 15625;            //timer 1 period value
    T3CONbits.TON = 1;      //turn on
    IEC0bits.T3IE = 1;      //enable
}

/*
void config_int4_int(){
    OSCCONbits.SOSCEN = 0;
    IEC0bits.INT4IE = 0;
    TRISCbits.TRISC13 = 1;
    INT4Rbits.INT4R = 0b0111;
    
    IPC5bits.IC4EIP = 7;
    INTCONbits.INT4EP = 1;
    IFS0bits.INT4IF = 0;
    IEC0bits.INT4IE = 1;
}
*/

void config_general_int()
{
    __builtin_enable_interrupts();  //global interrupt enable
    PRISS = 0x76543210;
    INTCONbits.MVEC = 1;            //Multi vector interrupt 
}


/*
 * 
 */
int main(int argc, char** argv) {
    
    // Turn off things for Reset
    PMCONbits.ON = 0;
    ETHCON1bits.ON = 0;
    
    // Set up Reset and debug LEDS
    DEBUG1_TRIS = OUTPUT;
    DEBUG2_TRIS = OUTPUT;
    DEBUG3_TRIS = OUTPUT;
    RESET_TRIS = OUTPUT;
    
    DEBUG1 = DEBUG_OFF;
    DEBUG2 = DEBUG_OFF;
    DEBUG3 = DEBUG_OFF;
    RESET = RESET_OFF;
    
    // Initialize SD card
    SD_init();
    
    // Initialize Altimeter
    I2C4_init(200000);
    MPL3115A2_init();
    
    // Initialize KX222
    config_KX222();
    
    // Initialize ADF
    setup_adf_tx();
    write_BPL(0,payload_buff,TX_SIZE);                                          //Write test payload_buff to transmit buffer
    
    
    // Initialize BNO
    config_BNO055();
    
    
    // Start with the current packet set to Packet A
    current_packet = &packet_A;
    

    config_timer1_int();
    config_timer2_int();
    config_timer3_int();  
    config_general_int();
    
    while(1);
    
    SD_unmount();
    
    return (EXIT_SUCCESS);
}

